@@ -12,11 +12,17 @@ from __future__ import annotations
 
 
 import argparse
-import codecs
 import io
+import json
+import os
 import pathlib
 import sys
+# ruff: noqa: E402
+os.environ['PYDEVD_DISABLE_FILE_VALIDATION'] = '1'
 from . import converters
+from . import err
+from . import util
+from .codebraid_defaults import CodebraidDefaults
 from .version import __version__ as version
 
 
@@ -81,6 +87,16 @@ def main():
                                     'For Jupyter kernels, also show  errors and a summary of rich output. '
                                     'Output still appears in the document as normal. '
                                     'Individual sessions can override this by setting live_output=false in the document.')
+    parser_pandoc.add_argument('--no-execute', action='store_true',
+                               help='Disable code execution.  Only load code output from cache, if it exists.')
+    parser_pandoc.add_argument('--only-code-output', metavar='FORMAT',
+                               help='Write code output to stdout in the specified format. '
+                                    'Cached output is written immediately. '
+                                    'Output from execution is written as soon as it becomes available. '
+                                    'No document is created. '
+                                    'Options --to and --output are only used (if at all) to inform code output formatting.')
+    parser_pandoc.add_argument('--stdin-json-header', action='store_true',
+                               help='Treat the first line of stdin as a header in JSON format containing data about stdin such as file origin.')
     parser_pandoc.add_argument('files', nargs='*', metavar='FILE',
                                help="Files (multiple files are allowed for formats supported by Pandoc)")
     for opts_or_long_opt, narg in PANDOC_OPTIONS.items():
@@ -120,7 +136,7 @@ def main():
 
 def pandoc(args):
     # Stay consistent with Pandoc's requirement of UTF-8
-    sys.stdin = codecs.getreader('utf_8_sig')(sys.stdin.buffer, 'strict')
+    sys.stdin = io.TextIOWrapper(sys.stdin.buffer, encoding='utf_8_sig')
 
     other_pandoc_args = []
     if vars(args).get('--defaults') is not None:
@@ -147,18 +163,47 @@ def pandoc(args):
 
     if not args.files or (len(args.files) == 1 and args.files[0] == '-'):
         paths = None
+        string_origins = None
         try:
             strings = sys.stdin.read()
         except UnicodeDecodeError as e:
-            sys.exit('Input must be UTF-8:\n{0}'.format(e))
+            sys.exit(f'Input must be UTF-8:\n{e}')
+        if args.stdin_json_header:
+            stdin_lines = util.splitlines_lf(strings)
+            if not stdin_lines:
+                sys.exit('Missing data for --stdin-json-header')
+            if len(stdin_lines) == 1:
+                stdin_lines.append('')
+            json_header_str = stdin_lines[0]
+            try:
+                json_header = json.loads(json_header_str)
+            except Exception as e:
+                sys.exit(f'Invalid data for --stdin-json-header:\n{e}')
+            if not isinstance(json_header, dict):
+                sys.exit(f'Invalid data for --stdin-json-header: expected dict')
+            json_header_origins = json_header.get('origins')
+            if json_header_origins is not None:
+                if not isinstance(json_header_origins, list):
+                    sys.exit('Invalid data for --stdin-json-header field "string_origins"')
+                start_line = 1
+                strings = []
+                string_origins = []
+                for entry in json_header_origins:
+                    origin_name = entry['path']
+                    origin_length = entry['lines']
+                    string_origins.append(origin_name)
+                    strings.append('\n'.join(stdin_lines[start_line:start_line+origin_length]))
+                    start_line += origin_length
+            else:
+                strings = '\n'.join(stdin_lines[1:])
     else:
         paths = args.files
         strings = None
+        string_origins = None
 
-    code_defaults = {}
-    session_defaults = {}
+    codebraid_defaults = CodebraidDefaults()
     if args.live_output:
-        session_defaults['live_output'] = args.live_output
+        codebraid_defaults['live_output'] = args.live_output
 
     preview = False
     other_pandoc_args_at_load = None
@@ -177,25 +222,31 @@ def pandoc(args):
             # There is also a check for this in `converter.convert()`, but should
             # fail early.
             sys.exit(f'File "{args.output}" already exists (to replace it, add option "--overwrite")')
-    with converters.PandocConverter(
-        paths=paths,
-        strings=strings,
-        from_format=args.from_format,
-        pandoc_file_scope=args.pandoc_file_scope,
-        no_cache=args.no_cache,
-        cache_path=args.cache_dir,
-        code_defaults=code_defaults,
-        session_defaults=session_defaults,
-        other_pandoc_args_at_load=other_pandoc_args_at_load,
-    ) as converter:
-        converter.convert(
-            to_format=args.to_format,
-            standalone=args.standalone,
-            output_path=output_path,
-            overwrite=args.overwrite,
-            other_pandoc_args=other_pandoc_args
-        )
-        exit_code = converter.exit_code
+    try:
+        with converters.PandocConverter(
+            paths=paths,
+            strings=strings,
+            string_origins=string_origins,
+            from_format=args.from_format,
+            pandoc_file_scope=args.pandoc_file_scope,
+            no_cache=args.no_cache,
+            cache_path=args.cache_dir,
+            codebraid_defaults=codebraid_defaults,
+            other_pandoc_args_at_load=other_pandoc_args_at_load,
+            no_execute=args.no_execute,
+            only_code_output=args.only_code_output,
+        ) as converter:
+            if not args.only_code_output:
+                converter.convert(
+                    to_format=args.to_format,
+                    standalone=args.standalone,
+                    output_path=output_path,
+                    overwrite=args.overwrite,
+                    other_pandoc_args=other_pandoc_args
+                )
+            exit_code = converter.exit_code
+    except err.YAMLMetadataError as e:
+        sys.exit(str(e))
 
     sys.exit(exit_code)
 
@@ -234,6 +285,7 @@ PANDOC_OPTIONS  = {
     '--resource-path': 1,
     '--request-header': 1,
     '--abbreviations': 1,
+    '--embed-resources': 0,
     '--self-contained': 0,
     '--html-q-tags': 0,
     '--ascii': 0,
